/**************************************************************

NEC IR decoder programma - Bart Huyskens - SJI Schoten - 23/04/2013

Sensor: TSOP1236 - aangesloten aan PIN RB0 (Externe interrupt)
deze sensor inverteert wel het resultaat... rekening mee houden - reactie op dalende flank!!

Timer 1 wordt gebruikt om de tijd tussen twee dalende flanken te meten

- Xtal: 19660800
- Prescaler: 8 - increment periode= 1.627usec

situaties:
0 - TMR1 overflow (interrupt) - NO KEY PRESSED +/-== 65536
1 - 13.5 msec tussen 2 neg flanken - START 		(8294 x 1.627usec) 	7500<time<9000
2 - 11.25 msec tussen 2 neg flanken - REPEAT 	(6912 x 1.627usec)	5000<time<7499
3 - 2.25 msec tussen 2 neg flanken - "1" 		(1382 x 1.627usec)	1050<time<1700
4 - 1.125 msec tussen 2 neg flanken - "0" 		(691 x 1.627usec)	300<time<1000	

***************************************************************/
#include <htc.h> 
#include "HITECH_LCD.h"		// include LCD library
#define _XTAL_FREQ  19660800  

__CONFIG(0x2FF2); // external HS Xtal 19660800
__CONFIG(0x3FFF);

// global variables - for use in all functions
unsigned int TMR1HCOPY, TMR1LCOPY, TMR1X; 
unsigned char situatie, START = 0;
unsigned char x,BIT, ADRES,ADRESINV,DATA, DATAINV = 0;

void main()
{

		TRISD = 0x00;				// leds aan PORTD output
		TRISB = 0x01;				// RB0 = input (interrupt)
		ANSEL = 0x00;				// all pins digital (not analog)
		ANSELH = 0x00;				// all pins digital (not analog)
        
		T1CON = 0b00110001;			//TMR1 - prescaler /8 - internal CLK - Start - timer loopt tegen 614.400Hz
		TMR1IE = 1;					//TMR1 interrupt enable
		PEIE = 1; 					// Peripheral int enable

		OPTION_REG = 0b00000000;    // INT on BO on Falling edge (Bit 6) (voor oudere versies is dit OPTION) OPTION_REG = 0b01000000; 
		INTE = 1;           		// enable External int on B0
		
        GIE = 1;            		// enable global interrupts

        LCD_Start();		
		LCD_Clear();						
		LCD_PrintString("NEC IR DECODER");
		for (char s = 0; s<6; s++)__delay_ms(250);		// show start message	
                              
    while(1) 
	{
		LCD_Clear();
		LCD_PrintString("ADR:");				
		LCD_PrintNumber(ADRES);			
		LCD_PrintString(" ");
		LCD_Cursor(8, 0);
		LCD_PrintString("/ADR:");		
		LCD_PrintNumber(ADRESINV);		
	
		
		LCD_Cursor(0, 1);
		LCD_PrintString("DAT:");			
		LCD_PrintNumber(DATA);			
		LCD_PrintString(" ");
		LCD_Cursor(8, 1);
		LCD_PrintString("/DAT:");			
		LCD_PrintNumber(DATAINV);	

		__delay_ms(50);		// flikkering LCD wegwerken
    } 
}

/***** INTERRUPT SERVICE ROUTINE *****/

void interrupt isr(void)  
{
if (TMR1IF) 				// int op TMR1 overflow
	{						// geeft aan dat er geen knop is ingedrukt
	TMR1IF = 0;				// clear TMR1 int flag
	situatie = 0;			// situatie 0 - zie boven
	TMR1H = 0x00;			// clear TMR1L
	TMR1L = 0x00;			// clear TMR1H
	}

else if (INTF)// interrupt op RB0 (dalende flank op RB0)
	{
	INTF = 0;				// Clear RB0 INT flag
	TMR1LCOPY = TMR1L;		// copieer TMR1L
	TMR1HCOPY = TMR1H; 		// copieer TMR1H
	TMR1H = 0x00;			// clear TMR1L
	TMR1L = 0x00;			// clear TMR1H
	TMR1X = ((TMR1HCOPY << 8)+(TMR1LCOPY)); // 16 bit samenstelling


		if ((TMR1X > 7500)&(TMR1X < 9000))  // detect situatie 1: START
		{
		START = 1;
		x = 0;
		situatie = 1;
		ADRES = 0;
		ADRESINV = 0;
		DATA = 0;
		DATAINV = 0;
		}
		
		else if ((TMR1X > 5000)&(TMR1X < 7499)) // detect situatie 2: REPEAT
		{
		situatie = 2; // repeat
		RD0 = ~RD0;
		}
	
		else if ((TMR1X > 1050)&(TMR1X < 1700)) // detect situatie 3: "1"
		{
		situatie = 3; //"1"
		BIT = 1;
		}
	
		else if ((TMR1X > 300)&(TMR1X < 1000)) // detect situatie 4: "0"
		{
		situatie = 4; //"0"
		BIT = 0;
		}
	
		else situatie = 5;  // alle andere situaties (zou niet mogen voorkomen)

	


		if ((situatie == 3)||(situatie == 4))  // als er een "1" of een "0" binnenkomt
		{
			if (x<8) // Bit by Bit laden van Byte ADRES
					{
					ADRES = ADRES | BIT;
					if (x < 7) ADRES = ADRES << 1;  // laatste bit mag niet geshift worden
					x++;
					}
			else if (x<16)// Bit by Bit laden van Byte ADRESINV
					{
					ADRESINV = ADRESINV | BIT;
					if (x< 15) ADRESINV = ADRESINV << 1; // laatste bit mag niet geshift worden
					x++;
					}
			else if (x<24)// Bit by Bit laden van Byte DATA
					{
					DATA = DATA | BIT;
					if (x< 23) DATA = DATA << 1; // laatste bit mag niet geshift worden
					x++;
					}
			else if (x<32)// Bit by Bit laden van Byte DATAINV
					{
					DATAINV = DATAINV | BIT;
					if (x< 31) DATAINV = DATAINV << 1; // laatste bit mag niet geshift worden
					x++;
					}
		}
	}
}
